<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2010 Different Solutions GmbH <info@different-solutions.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

require_once(t3lib_extMgm::extPath('ds_vtiger_crm', 'Http/Client.php'));
require_once(t3lib_extMgm::extPath('ds_vtiger_crm', 'Zend/Json.php'));

/**
 * VtigerConnector
 */
class VtigerConnector
{
    /**
     * @var \HttpClient
     */
    protected $_client = null;

    /**
     * @var array module|action|username|userid|accesskey|sessionkey
     */
    protected $_meta = array();

    /**
     * @var array
     */
    protected $_preparedData = array();

    /**
     * Constructor
     * @param string $target
     * @param string $username
     * @param string $accesskey
     * @param string $module
     * @param string $action create|update|delete|retrieve
     */
    public function __construct($target, $username, $accesskey, $module, $action='create') {
        $this->_client = new HttpClient($target);

        $this->_meta = array(
            'module'    => $module,
            'action'    => $action,
            'username'  => $username,
            'accesskey' => $accesskey
        );
    }

    /**
     * Connect to vtiger
     * @return bool
     */
    public function connectToVtiger() {
        $success = false;
        $token = $this->getChallengeToken();

        if ($token != ''  && $this->loginUser($token)) {
            $success = true;
        }

        return $success;
    }

    /**
     * Disconnect from vtiger
     * @return bool
     */
    public function disconnectFromVtiger() {
        return $this->logoutUser();
    }

    /**
     * Get challenge token
     * @return string
     */
    public function getChallengeToken() {
        $token = '';

        $requestData = array(
            'operation' => 'getchallenge',
            'username'  => $this->_meta['username']
        );

        $response = $this->sendRequest($requestData, 'GET');

        if ($response['success']) $token = $response['result']['token'];

        return $token;
    }

    /**
     * Authenticate user on vtiger
     * Update $this->_meta information with userid and sessionkey
     * @param  $challenge_token
     * @return bool
     */
    public function loginUser($challenge_token) {
        $success = false;

        $requestData = array(
            'operation' => 'login',
            'username'  => $this->_meta['username'],
            'accessKey' => md5($challenge_token.$this->_meta['accesskey'])
        );

        $response = $this->sendRequest($requestData, 'POST'); // PUT

        if ($response['success']) {
            $this->_meta['userid']     = $response['result']['userId'];
            $this->_meta['sessionkey'] = $response['result']['sessionName'];
            $success = true;
        }

        return $success;
    }

    /**
     * Logout user
     * @return bool
     */
    public function logoutUser() {
       $success = false;

       $requestData = array(
           'operation'   => 'logout',
           'sessionName' => $this->_meta['sessionkey'],
       );

       $response = $this->sendRequest($requestData, 'POST'); // PUT

       if ($response['success']) $success = true;

       return $success;
    }

    /**
     * Prepare data for transfer
     * @param  array  $data
     * @param  int    $pid
     * @return bool
     */
    public function prepareDataForModule(&$data, $pid) {
        $success  = false;
        $tmp_data = array();
        $sql_uids = '';

        foreach ($data as $key => $value) {
            $uid = intval(str_replace(array('uid', '_'), '', $key));
            $tmp_data[$uid] = $value;
            $sql_uids .= $uid.',';
        }
        $sql_uids = substr($sql_uids, 0, -1);

        $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery(
            'uid, tx_dsvtigercrm_name',
            'tx_powermail_fields',
            'pid='.intval($pid).' and uid in('.$sql_uids.') and tx_dsvtigercrm_name!=""'
        );

        if ($GLOBALS['TYPO3_DB']->sql_num_rows($res) != 0) {
            while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
                $preparedValue = $this->_preparedData[$row['tx_dsvtigercrm_name']];
                $this->_preparedData[$row['tx_dsvtigercrm_name']] = trim($preparedValue . ' ' . $tmp_data[$row['uid']]);
            }

            $userid = $this->_preparedData['assigned_user_id'];
            if (isset($userid) && trim($userid) != '') {
                if (is_numeric($userid)) $userid = '19x' . $userid;
            } else {
                $userid = ($this->_meta['userid']) ? $this->_meta['userid'] : '19x1';
            }
            $this->_preparedData['assigned_user_id'] = $userid;

            $success = true;
        }

        return $success;
    }

    /**
     * Send data for module to vtiger and execute the operation
     * @return bool
     */
    public function sendDataToModule() {
        $success = false;

        $requestData = array(
            'operation'   => $this->_meta['action'],
            'sessionName' => $this->_meta['sessionkey'],
            'element'     => Zend_JSON::encode($this->_preparedData),
            'elementType' => $this->_meta['module']
        );

        $response = $this->sendRequest($requestData, 'POST'); // PUT

        if ($response['success']) {
            $success = true;
        }

        return $success;
    }

    /**
     * Send request to vtiger
     * @param  array $requestData
     * @param  string $requestAction POST|GET|PUT|DELETE
     * @return array
     */
    protected function sendRequest(&$requestData, $requestAction) {
        $this->_client->setVerb($requestAction);

        if ($requestAction == 'GET') {
            $this->_client->buildGetParameter($requestData);
        } else {
            $this->_client->buildPostBody($requestData);
        }

        $this->_client->execute();

        return $this->getLastResponse();
    }

    /**
     * Show decoded response
     * @return array
     */
    public function getLastResponse() {
        return Zend_Json::decode($this->_client->getResponseBody());
    }

    /**
     * Show meta data
     * @return array
     */
    public function getMetaData() {
        return $this->_meta;
    }

    /**
     * Show prepared data
     * @return array
     */
    public function getPreparedData() {
        return $this->_preparedData;
    }

    /**
     * Destructor
     */
    public function __destruct() {
        unset($this->_client);
    }
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ds_vtiger_crm/tx_dsvtigercrm_rest.php'])    {
    include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ds_vtiger_crm/tx_dsvtigercrm_rest.php']);
}

?>